#!/usr/bin/python

import sys
import shutil
import os
import charmhelpers.core.hookenv as hookenv
import charmhelpers.fetch as fetch
import charmhelpers.core.host as host
from charmhelpers.core.hookenv import UnregisteredHookError
from utils import (
    render_template,
)

NTP_CONF = '/etc/ntp.conf'
NTP_CONF_ORIG = '{}.orig'.format(NTP_CONF)

hooks = hookenv.Hooks()


@hooks.hook('install')
def install():
    fetch.apt_update(fatal=True)
    fetch.apt_install(["ntp"], fatal=True)
    shutil.copy(NTP_CONF, NTP_CONF_ORIG)
    hookenv.open_port(123, protocol="UDP")


@hooks.hook('upgrade-charm')
@hooks.hook('config-changed')
@hooks.hook('peer-relation-changed')
@hooks.hook('peer-relation-departed')
@host.restart_on_change({NTP_CONF: ['ntp']})
def write_config():
    source = hookenv.config('source')
    remote_sources = []
    peers = []
    local_time = False
    if source and len(source) > 0:
        if source == "LOCAL":
            local_time = True
        else:
            for s in source.split(' '):
                remote_sources.append({"name": s})

    for relid in hookenv.relation_ids('peer'):
        for unit in hookenv.related_units(relid):
            u_addr = hookenv.relation_get(attribute='private-address',
                                          unit=unit, rid=relid)
            peers.append({'name': u_addr})

    if local_time or len(remote_sources) > 0 or len(peers) > 0:
        ntp_context = {
            'local_time': local_time,
            'servers': remote_sources,
            'peers': peers
            }
        with open(NTP_CONF, "w") as ntpconf:
            ntpconf.write(render_template(os.path.basename(NTP_CONF),
                                          ntp_context))
    else:
        shutil.copy(NTP_CONF_ORIG, NTP_CONF)


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        hookenv.log('Unknown hook {} - skipping.'.format(e))
